"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Production = void 0;
const node_1 = require("./node");
const production_node_1 = require("./production-node");
const error_1 = require("../error");
class Production extends node_1.Node {
    constructor(location, nodes) {
        super(location);
        this.nodes = nodes;
    }
    is_epsilon() {
        return this.nodes.length === 0;
    }
    get terminals_maps() {
        if (this._terminals_maps === undefined) {
            this._terminals_maps = [];
            for (let i = 0; i < this.nodes.length; ++i) {
                const node = this.nodes[i];
                const terminals = [];
                let epsilon_found = false;
                if (node.type === production_node_1.ProductionNodeType.TERMINAL_USAGE) {
                    terminals.push(node.reference);
                }
                else if (node.type === production_node_1.ProductionNodeType.VARIABLE_USAGE) {
                    const variable = node.reference;
                    for (const map of variable.terminals_maps) {
                        for (const terminal of map.terminals) {
                            terminals.push(terminal);
                        }
                    }
                    epsilon_found = variable.epsilon;
                }
                else {
                    throw new error_1.InternalError(`unknown production node type \`${node.type}\``);
                }
                this._terminals_maps.push({
                    terminals,
                    nodes: this.nodes.slice(i)
                });
                if (!epsilon_found)
                    break;
            }
        }
        return this._terminals_maps;
    }
    static create_epsilon(locatable) {
        return new Production(locatable.get_location(), []);
    }
}
exports.Production = Production;
//# sourceMappingURL=production.js.map