#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const args_1 = __importDefault(require("./args"));
const pipeline_1 = __importDefault(require("./pipeline"));
const error_1 = require("./error");
try {
    const args = args_1.default.parse();
    const pipeline = new pipeline_1.default(args);
    pipeline.run();
}
catch (error) {
    if (error instanceof error_1.ArgsError) {
        console.log(error.message);
        process.exit(1);
    }
    else if (error instanceof error_1.CodeError) {
        console.log(error.message);
        process.exit(2);
    }
    else if (error instanceof error_1.InternalError) {
        console.error(error.stack || error.message);
        process.exit(3);
    }
    else {
        const _err = error;
        console.error(`UNHANDLED ERROR: ${_err.stack || _err.message || String(_err)}`);
        process.exit(4);
    }
}
//# sourceMappingURL=main.js.map