"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NonDeterministicStatesMap = void 0;
const error_1 = require("../error");
class NonDeterministicStatesMap {
    constructor(context, initial_states) {
        this.context = context;
        this.initial_states_ids = new Set();
        this.virtual_to_initial_states_map = new Map();
        this.map = new Map();
        for (let state of initial_states) {
            this.initial_states_ids.add(state.id);
            this.map.set(state.id.toString(), state);
        }
    }
    get_or_create(non_deterministic_states) {
        const states_ids = new Set();
        for (const state of non_deterministic_states) {
            if (this.initial_states_ids.has(state.id)) {
                states_ids.add(state.id);
                continue;
            }
            const sub_initial_states = this.virtual_to_initial_states_map.get(state.id);
            if (sub_initial_states !== undefined) {
                sub_initial_states.forEach(id => states_ids.add(id));
                continue;
            }
            throw new error_1.InternalError(`One of provided states (${state.id}) is not initial and even not virtual.`);
        }
        const key = [...states_ids].sort((a, b) => a - b).join(',');
        let value = this.map.get(key);
        if (value !== undefined)
            return value;
        value = this.context.create_new_state();
        const machine_ids = [];
        for (let state of non_deterministic_states) {
            if (state.is_final) {
                if (state.machine_id !== undefined)
                    machine_ids.push(state.machine_id);
                value.is_final = true;
            }
            value.add_transitions(...state.transitions);
        }
        if (machine_ids.length !== 0) {
            value.machine_id = Math.min(...machine_ids);
        }
        this.virtual_to_initial_states_map.set(value.id, [...states_ids]);
        this.map.set(key, value);
        return value;
    }
}
exports.NonDeterministicStatesMap = NonDeterministicStatesMap;
//# sourceMappingURL=non-deterministic-states-map.js.map