"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SourceReader = void 0;
const location_1 = require("./location");
class SourceReader {
    constructor(reader, options) {
        this.reader = reader;
        this.line = options?.location_offset?.line || 1;
        this.column = (options?.location_offset?.column || 1) - 1;
        this.current = '\a';
        this.cache = null;
        this.file = options?.file || '<unknown>';
    }
    read() {
        if (this.current === '')
            return '';
        if (this.current === '\n') {
            this.line += 1;
            this.column = 1;
        }
        else {
            this.column += 1;
        }
        if (this.cache === null) {
            this.current = this.reader.read();
        }
        else {
            this.current = this.cache;
            this.cache = null;
        }
        return this.current;
    }
    revoke() {
        this.cache = this.current;
        if (this.current === '\n') {
            this.column -= 1;
            this.current = '\a';
        }
        else if (this.current !== '') {
            this.column -= 1;
        }
    }
    get_point() {
        return new location_1.Point(this.line, this.column);
    }
}
exports.SourceReader = SourceReader;
//# sourceMappingURL=source-reader.js.map