"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeBuilder = void 0;
class CodeBuilder {
    constructor(options) {
        this.fragments = [];
        this.single_indentation = ' '.repeat(options?.indentation_count || 4);
        this.indentation_level = 0;
        this.should_print_indentation = true;
    }
    indent() {
        this.indentation_level += 1;
        return this;
    }
    unindent() {
        if (this.indentation_level === 0)
            throw new Error('can\'t decrease the indentation that is already 0');
        this.indentation_level -= 1;
        return this;
    }
    wrap_indentation(callback) {
        try {
            this.indent();
            callback(this);
        }
        finally {
            this.unindent();
        }
        return this;
    }
    print(line) {
        if (this.should_print_indentation) {
            this.fragments.push(this.single_indentation.repeat(this.indentation_level));
            this.should_print_indentation = false;
        }
        this.fragments.push(line);
        return this;
    }
    println(line) {
        return this.print(line).newline();
    }
    newline() {
        this.fragments.push('\n');
        this.should_print_indentation = true;
        return this;
    }
    ensure_empty_line() {
        const match = this.to_string().match(/\n*$/)?.[0] || '';
        const missing = Math.max(2 - match.length, 0);
        for (let i = 0; i < missing; i += 1)
            this.newline();
        return this;
    }
    to_string() {
        const reduced = this.fragments.join('');
        this.fragments = [reduced];
        return reduced;
    }
}
exports.CodeBuilder = CodeBuilder;
//# sourceMappingURL=code-builder.js.map